﻿Partial Public Class MainPage
    Inherits PhoneApplicationPage
    Private Const DECELERATION As Double = 250 ' pixels per second squared

    Private isDragging As Boolean
    Private flickVelocity, flickAngle As Double
    Private absoluteEllipseLocation As Point
    Private flickTickTime As Date

    Public Sub New()
        InitializeComponent()
    End Sub

    Private Sub OnGestureListenerDragStarted(ByVal sender As Object, ByVal args As DragStartedGestureEventArgs)
        isDragging = (args.OriginalSource Is ellipsePath)

        If isDragging Then
            RemoveHandler CompositionTarget.Rendering, AddressOf OnCompositionTargetRendering
        End If
    End Sub

    Private Sub OnGestureListenerDragDelta(ByVal sender As Object, ByVal args As DragDeltaGestureEventArgs)
        If isDragging Then
            ellipseGeometry.Center = New Point(ellipseGeometry.Center.X + args.HorizontalChange,
                                               ellipseGeometry.Center.Y + args.VerticalChange)
        End If
    End Sub

    Private Sub OnGestureListenerDragCompleted(ByVal sender As Object, ByVal args As DragCompletedGestureEventArgs)
        isDragging = False
    End Sub

    Private Sub OnGestureListenerFlick(ByVal sender As Object, ByVal args As FlickGestureEventArgs)
        If isDragging OrElse args.OriginalSource Is ellipsePath Then
            flickVelocity = Math.Sqrt(Math.Pow(args.HorizontalVelocity, 2) +
                                      Math.Pow(args.VerticalVelocity, 2))
            flickAngle = args.Angle
            flickTickTime = Date.Now
            absoluteEllipseLocation = ellipseGeometry.Center

            AddHandler CompositionTarget.Rendering, AddressOf OnCompositionTargetRendering
        End If
    End Sub

    Private Sub OnCompositionTargetRendering(ByVal sender As Object, ByVal args As EventArgs)
        Dim eventTime = Date.Now
        Dim secondsElapsed = (eventTime.Subtract(flickTickTime)).TotalSeconds
        flickTickTime = eventTime

        Dim delta = flickVelocity * secondsElapsed
        Dim horzDelta = delta * Cos(flickAngle)
        Dim vertDelta = delta * Sin(flickAngle)

        ' Calculate new location of ellipse
        absoluteEllipseLocation = New Point(absoluteEllipseLocation.X + horzDelta,
                                            absoluteEllipseLocation.Y + vertDelta)

        ' Bounce it off the edges
        Dim bounceSize As New Size(ContentPanel.ActualWidth - 2 * ellipseGeometry.RadiusX,
                                   ContentPanel.ActualHeight - 2 * ellipseGeometry.RadiusY)

        Dim x = Math.Abs((absoluteEllipseLocation.X - ellipseGeometry.RadiusX) Mod (2 * bounceSize.Width))
        If x > bounceSize.Width Then
            x = 2 * bounceSize.Width - x
        End If
        x += ellipseGeometry.RadiusX

        Dim y = Math.Abs((absoluteEllipseLocation.Y - ellipseGeometry.RadiusY) Mod (2 * bounceSize.Height))
        If y > bounceSize.Height Then
            y = 2 * bounceSize.Height - y
        End If
        y += ellipseGeometry.RadiusY

        ' Set the new ellipse center
        ellipseGeometry.Center = New Point(x, y)

        ' Decrease velocity by deceleration
        flickVelocity -= secondsElapsed * DECELERATION

        If flickVelocity <= 0 Then
            RemoveHandler CompositionTarget.Rendering, AddressOf OnCompositionTargetRendering
        End If
    End Sub

    Private Function Sin(ByVal angle As Double) As Double
        Return Math.Sin(Math.PI * angle / 180)
    End Function

    Private Function Cos(ByVal angle As Double) As Double
        Return Math.Cos(Math.PI * angle / 180)
    End Function
End Class