﻿Partial Public Class MainPage
    Inherits PhoneApplicationPage
    Private isDragging As Boolean
    Private isPinching As Boolean
    Private ptPinchPositionStart As Point

    Public Sub New()
        InitializeComponent()
    End Sub

    Private Sub OnGestureListenerDragStarted(ByVal sender As Object, ByVal args As DragStartedGestureEventArgs)
        isDragging = args.OriginalSource Is image
    End Sub

    Private Sub OnGestureListenerDragDelta(ByVal sender As Object, ByVal args As DragDeltaGestureEventArgs)
        If isDragging Then
            translateTransform.X += args.HorizontalChange
            translateTransform.Y += args.VerticalChange
        End If
    End Sub

    Private Sub OnGestureListenerDragCompleted(ByVal sender As Object, ByVal args As DragCompletedGestureEventArgs)
        If isDragging Then
            TransferTransforms()
            isDragging = False
        End If
    End Sub

    Private Sub OnGestureListenerPinchStarted(ByVal sender As Object, ByVal args As PinchStartedGestureEventArgs)
        isPinching = args.OriginalSource Is image

        If isPinching Then
            ' Set transform centers
            Dim ptPinchCenter = args.GetPosition(image)
            ptPinchCenter = previousTransform.Transform(ptPinchCenter)

            scaleTransform.CenterX = ptPinchCenter.X
            scaleTransform.CenterY = ptPinchCenter.Y

            rotateTransform.CenterX = ptPinchCenter.X
            rotateTransform.CenterY = ptPinchCenter.Y

            ptPinchPositionStart = args.GetPosition(Me)
        End If
    End Sub

    Private Sub OnGestureListenerPinchDelta(ByVal sender As Object, ByVal args As PinchGestureEventArgs)
        If isPinching Then
            ' Set scaling
            scaleTransform.ScaleX = args.DistanceRatio
            scaleTransform.ScaleY = args.DistanceRatio

            ' Optionally set rotation
            If allowRotateCheckBox.IsChecked.Value Then
                rotateTransform.Angle = args.TotalAngleDelta
            End If

            ' Set translation
            Dim ptPinchPosition = args.GetPosition(Me)
            translateTransform.X = ptPinchPosition.X - ptPinchPositionStart.X
            translateTransform.Y = ptPinchPosition.Y - ptPinchPositionStart.Y
        End If
    End Sub

    Private Sub OnGestureListenerPinchCompleted(ByVal sender As Object, ByVal args As PinchGestureEventArgs)
        If isPinching Then
            TransferTransforms()
            isPinching = False
        End If
    End Sub

    Private Sub TransferTransforms()
        previousTransform.Matrix = Multiply(previousTransform.Matrix, currentTransform.Value)

        ' Set current transforms to default values
        scaleTransform.ScaleY = 1
        scaleTransform.ScaleX = scaleTransform.ScaleY
        scaleTransform.CenterY = 0
        scaleTransform.CenterX = scaleTransform.CenterY

        rotateTransform.Angle = 0
        rotateTransform.CenterY = 0
        rotateTransform.CenterX = rotateTransform.CenterY

        translateTransform.Y = 0
        translateTransform.X = translateTransform.Y
    End Sub

    Private Function Multiply(ByVal A As Matrix, ByVal B As Matrix) As Matrix
        Return New Matrix(A.M11 * B.M11 + A.M12 * B.M21,
                          A.M11 * B.M12 + A.M12 * B.M22,
                          A.M21 * B.M11 + A.M22 * B.M21,
                          A.M21 * B.M12 + A.M22 * B.M22,
                          A.OffsetX * B.M11 + A.OffsetY * B.M21 + B.OffsetX,
                          A.OffsetX * B.M12 + A.OffsetY * B.M22 + B.OffsetY)
    End Function
End Class